@@ -13,7 +13,7 @@
 #include "sg_cmds_basic.h"
 
 /*
-*  Copyright (C) 2000-2008 D. Gilbert
+*  Copyright (C) 2000-2009 D. Gilbert
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -26,7 +26,7 @@
 
 */
 
-static char * version_str = "1.30 20080805";
+static char * version_str = "1.32 20090826";
 
 #define DEF_ALLOC_LEN (1024 * 4)
 #define DEF_6_ALLOC_LEN 252
@@ -530,6 +530,7 @@ static struct page_code_desc pc_desc_disk[] = {
     {0x10, 0x0, "XOR control"},
     {0x1a, 0xf1, "ATA Power condition"},
     {0x1c, 0x1, "Background control"},
+    {0x1c, 0x2, "Thin provisioning"},
 };
 
 static struct page_code_desc pc_desc_tape[] = {
@@ -577,7 +578,7 @@ static struct page_code_desc pc_desc_rbc[] = {
     {0x6, 0x0, "RBC device parameters"},
 };
 
-static struct page_code_desc pc_desc_adt[] = {
+static struct page_code_desc pc_desc_adc[] = {
     /* {0xe, 0x0, "ADC device configuration"}, */
     {0xe, 0x1, "Target device"},
     {0xe, 0x2, "DT device primary port"},
@@ -618,8 +619,8 @@ mode_page_cs_table(int scsi_ptype, int * size)
             *size = sizeof(pc_desc_rbc) / sizeof(pc_desc_rbc[0]);
             return &pc_desc_rbc[0];
         case PDT_ADC:       /* automation device/interface */
-            *size = sizeof(pc_desc_adt) / sizeof(pc_desc_adt[0]);
-            return &pc_desc_adt[0];
+            *size = sizeof(pc_desc_adc) / sizeof(pc_desc_adc[0]);
+            return &pc_desc_adc[0];
     }
     *size = 0;
     return NULL;
@@ -644,10 +645,10 @@ static struct page_code_desc pc_desc_t_sas[] = {
     {0x19, 0x0, "Protocol specific port (SAS)"},
     {0x19, 0x1, "Phy control and discover (SAS)"},
     {0x19, 0x2, "Shared port control (SAS)"},
-    {0x19, 0x3, "SAS-2 phy"},
+    {0x19, 0x3, "Enhanced phy control (SAS)"},
 };
 
-static struct page_code_desc pc_desc_t_adt[] = {
+static struct page_code_desc pc_desc_t_adc[] = {
     {0xe, 0x1, "Target device"},
     {0xe, 0x2, "DT device primary port"},
     {0xe, 0x3, "Logical unit"},
@@ -670,8 +671,8 @@ mode_page_transp_table(int t_proto, int * size)
             *size = sizeof(pc_desc_t_sas) / sizeof(pc_desc_t_sas[0]);
             return &pc_desc_t_sas[0];
         case TPROTO_ADT:
-            *size = sizeof(pc_desc_t_adt) / sizeof(pc_desc_t_adt[0]);
-            return &pc_desc_t_adt[0];
+            *size = sizeof(pc_desc_t_adc) / sizeof(pc_desc_t_adc[0]);
+            return &pc_desc_t_adc[0];
     }
     *size = 0;
     return NULL;
